--*********************************************************
--* FILE  : MIOS_USB.VHD
--* Author: Jack Fried
--*
--* Last Modified: 5/19/2005
--*  
--* Description: interface to the cypress USB FIFO's
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY USB_cntl IS

	PORT
	(
		clk         : IN STD_LOGIC;				-- clock
		rst         : IN STD_LOGIC;				-- state machine reset
		c_FD        : INOUT STD_LOGIC_VECTOR(15 downto 0);	-- cypress slave fifo data		
		c_EP2FF     : IN STD_LOGIC;				-- cypress full flag for fifo endpoint 2  - flag_a	  
		c_EP6EF   	: IN STD_LOGIC;				-- cypress empty flag for fifo endpoint 6 - flag_b
		c_EP8FF    	: IN STD_LOGIC;				-- cypress full flag for fifo endpoint 8  - flag_c
		c_SLOE		: OUT STD_LOGIC;   			-- cypress slave fifo output enable
		c_SLRD	  	: OUT STD_LOGIC;			-- cypress slave fifo read
		c_SLWR     	: OUT STD_LOGIC;			-- cypress slave fifo write
		c_FADDR0	: OUT STD_LOGIC;			-- cypress slave fifo address 0 
		c_FADDR1	: OUT STD_LOGIC;			-- cypress slave fifo address 1
		c_PKTEND	: OUT STD_LOGIC;			-- cypress slave fifo packet end 
		F1_rclk		: OUT STD_LOGIC;			-- FIFO1 
		F2_wclk		: OUT STD_LOGIC;			-- FIFO2
		F3_rclk		: OUT STD_LOGIC;			-- FIFO3
		F1_d		: IN  STD_LOGIC_VECTOR(15 downto 0);	-- FIFO streaming data out		
		F2_d		: OUT STD_LOGIC_VECTOR(15 downto 0);	-- FIFO control data in 		
		F3_d		: IN  STD_LOGIC_VECTOR(15 downto 0);	-- FIFO control data out
		F1_rdreq	: OUT STD_LOGIC;			-- FIFO1  read  request
		F2_wrreq	: OUT STD_LOGIC;			-- FIFO2  write request
		F3_rdreq	: OUT STD_LOGIC;			-- FIFO3  read  request
		F1_empty    : IN STD_LOGIC;			-- FIFO1  empty flag
		F2_full 	: IN STD_LOGIC;			-- FIFO2  full flag
		F3_empty	: IN STD_LOGIC				-- FIFO3  empty flag
		
	);
	-- {{ALTERA_IO_END}} DO NOT REMOVE THIS LINE!
	
END USB_cntl;


ARCHITECTURE behavior OF USB_cntl IS

  type state_type is (S_idle,S_start_EP8,S_start_EP6,S_start_EP2,S_EP6_FWR ,S_EP6_CRD,S_EP8_FRD,S_EP8_FRD2,S_EP8_PKTEND,S_EP2_FRD,S_EP2_FRD2,S_EP2_PKTEND);
  signal state     : state_type;
  signal reg_cntl  : STD_LOGIC;  -- when set data will flow to ep6 or ep8
  signal reg_io    : STD_LOGIC;  -- when set data will flow to ep8
  signal ep2_cnt   : STD_LOGIC_VECTOR(11 downto 0);
  signal ep8_cnt   : STD_LOGIC_VECTOR(11 downto 0);
  signal PKTEND2_cnt   : STD_LOGIC_VECTOR(15 downto 0);
  signal PKTEND8_cnt   : STD_LOGIC_VECTOR(15 downto 0);
begin
			--	c_FADDR0		
			--	c_FADDR1 
		c_FADDR0	<= '0'  when (reg_io = '0')   else '1';	  -- set for ep6    else ep8		
		c_FADDR1    <= '0'  when (reg_cntl = '0') else '1';   -- set for ep2    else ep6 or ep8 
        c_FD		<= F1_d when ((reg_io = '0') and (reg_cntl= '0')) else 
                       F3_d when ((reg_io = '1') and (reg_cntl= '1')) else
                       (others => 'Z');
        F2_d		<=  c_FD;
 		F1_rclk		<=  clk;
		F2_wclk		<=  clk;
		F3_rclk		<=  clk;

  process(clk,rst) 
  begin
     if (rst = '1') then
		state <= S_idle;
		reg_io   <= '0';	
		reg_cntl <= '0';	
		c_PKTEND <= '1';
		c_SLOE	 <= '1';	
		c_SLRD	 <= '1';  	
		c_SLWR   <= '1';
		F1_rdreq <= '0';
		F2_wrreq <= '0';
		F3_rdreq <= '0';
		ep2_cnt  <= X"000";
		ep8_cnt  <= X"000";
		PKTEND2_cnt <= X"0000";
		PKTEND8_cnt <= X"0000";		
     elsif (clk'event AND clk = '0') then
        CASE state IS
     when S_IDLE =>	
		reg_io   <= '0';	
		reg_cntl <= '0';	
		c_PKTEND <= '1';
		c_SLOE	 <= '1';	
		c_SLRD	 <= '1';  	
		c_SLWR   <= '1';
		F1_rdreq <= '0';
		F2_wrreq <= '0';
		F3_rdreq <= '0';
		PKTEND2_cnt <= X"0000";
		PKTEND8_cnt <= X"0000";
		if      (F3_empty = '0') and (c_EP8FF = '1') then  state <= S_start_EP8;
		elsif   (c_EP6EF  = '1') and ( F2_full= '0') then  state <= S_start_EP6;
		elsif   (F1_empty = '0') and (c_EP2FF = '1') then  state <= S_start_EP2;
		ELSE  state <= S_idle;
		end if;
-----------------------------------------------------
	 when S_start_EP8 =>
			reg_io   <= '1';
			reg_cntl <= '1';
			c_SLWR   <= '1';			
			state <= S_EP8_FRD;
	 when S_start_EP6 =>
	        c_SLOE	 <= '0';
 			reg_io   <= '0';
			reg_cntl <= '1';
			c_SLWR   <= '1';
			state <= S_EP6_CRD;
	 when S_start_EP2 =>
  			reg_io   <= '0';
			reg_cntl <= '0';
			c_SLWR   <= '1';
			state <= S_EP2_FRD;		


-----------------------------------------------------	
     when S_EP6_CRD =>
		  if ((F2_full= '1') OR (c_EP6EF  = '0')) then  
			  state <= S_idle;
		  else 
			  c_SLRD   <= '0';
			  F2_wrreq <= '1';		
			  state <= S_EP6_FWR;
		  end if;
     when S_EP6_FWR =>
			  c_SLRD   <= '1';
			  F2_wrreq <= '0';			
		  if ((F2_full= '1') OR (c_EP6EF  = '0')) then  
			  state <= S_idle;
		  else 
			  state <= S_EP6_CRD;
		  end if;	
		
-------------------------------------------------------	

	     when S_EP8_FRD =>		-- start reading data from FIFO
		  PKTEND2_cnt <= X"0000";
		  if (c_EP6EF  = '1') and ( F2_full= '0') then  -- check if cmd data is avalible
			    F3_rdreq <= '0';
			    c_SLWR   <= '1';		
			    state <= S_start_EP6;
		  elsif (c_EP8FF = '0') then  	-- check to see if the cypress endpoint fifo is full
			    F3_rdreq <= '0';
			    c_SLWR   <= '1';
				ep8_cnt  <= X"000";	
			    state <= S_idle;			
		  elsif (F3_empty = '1') then  --  check to see if the altera fifo is empty
			    F3_rdreq <= '0';
			    c_SLWR   <= '1';
			    state <= S_EP8_PKTEND;
		  else 
			    c_SLWR   <= '1';
				ep8_cnt <= ep8_cnt +1;		-- count number of packets in endpoint fifo fifo
				if (ep8_cnt = X"100") then  -- 
				   F3_rdreq <= '0'; 
				   ep8_cnt  <= X"000";	
				   state <= S_idle;
				else 
				   F3_rdreq <= '1';	
				   state <= S_EP8_FRD2;					
				end if;
		  end if;		
	     when S_EP8_FRD2 =>   -- start writing data to endpoint  FIFO
		  if (c_EP6EF  = '1') and ( F2_full= '0') then  
			    F3_rdreq <= '0';
			    c_SLWR   <= '0';		
			    state <= S_start_EP6;
		  elsif (c_EP8FF = '0') then  
			    F3_rdreq <= '0';
			    c_SLWR   <= '1';
				ep8_cnt  <= X"000";	
			    state <= S_idle;			
		  elsif (F3_empty = '1') then  
			    F3_rdreq <= '0';
			    c_SLWR   <= '0';
			    state <= S_EP8_PKTEND;
		  else 
				ep8_cnt <= ep8_cnt +1;
			    c_SLWR   <= '0';				
				if (ep8_cnt = X"100") then 
				   F3_rdreq <= '0'; 
				   ep8_cnt  <= X"000";	
				   state <= S_idle;
				else 
				   F3_rdreq <= '1';
			       state <= S_EP8_FRD2;				
				end if;
		  end if;		
		
     when S_EP8_PKTEND =>   -- send partial packet
			c_SLWR   <= '1';
			F3_rdreq <= '0';			
			PKTEND8_cnt <= PKTEND8_cnt +1;
			if ((F3_empty = '0') and (c_EP8FF = '1')) then 
			    state <= S_EP8_FRD ;
			elsif (PKTEND8_cnt = X"0140") THEN  -- wait 128 clocks to see if more data arrives
				c_PKTEND <= '0';
			    ep8_cnt  <= X"000";
				state    <= S_idle;
			else
				state <= S_EP8_PKTEND;
			end if;
		
	
	--------------------------------------------------------------
		
     when S_EP2_FRD =>		-- start reading data from FIFO
		  PKTEND2_cnt <= X"0000";
		  if (F3_empty = '0') and (c_EP8FF = '1') then  -- check to see if there is cmd data to send
			    F1_rdreq <= '0';
			    c_SLWR   <= '1';
		        state <= S_start_EP8;			
		  elsif (c_EP6EF  = '1') and ( F2_full= '0') then  -- check if cmd data is avalible
			    F1_rdreq <= '0';
			    c_SLWR   <= '1';		
			    state <= S_start_EP6;
		  elsif (c_EP2FF = '0') then  	-- check to see if the cypress endpoint fifo is full
			    F1_rdreq <= '0';
			    c_SLWR   <= '1';
				ep2_cnt  <= X"000";	
			    state <= S_idle;			
		  elsif (F1_empty = '1') then  --  check to see if the altera fifo is empty
			    F1_rdreq <= '0';
			    c_SLWR   <= '1';
			    state <= S_EP2_PKTEND;
		  else 
			    c_SLWR   <= '1';
				ep2_cnt <= ep2_cnt +1;		-- count number of packets in endpoint fifo fifo
				if (ep2_cnt = X"100") then  -- 
				   F1_rdreq <= '0'; 
				   ep2_cnt  <= X"000";	
				   state <= S_idle;
				else 
				   F1_rdreq <= '1';	
				   state <= S_EP2_FRD2;					
				end if;
		  end if;
	     when S_EP2_FRD2 =>      -- start writing data to endpoint  FIFO
		  if (F3_empty = '0') and (c_EP8FF = '1') then  
			    F1_rdreq <= '0';
			    c_SLWR   <= '0';
		        state <= S_start_EP8;			
		  elsif (c_EP6EF  = '1') and ( F2_full= '0') then  
			    F1_rdreq <= '0';
			    c_SLWR   <= '0';		
			    state <= S_start_EP6;
		  elsif (c_EP2FF = '0') then  
			    F1_rdreq <= '0';
			    c_SLWR   <= '1';
				ep2_cnt  <= X"000";	
			    state <= S_idle;			
		  elsif (F1_empty = '1') then  
			    F1_rdreq <= '0';
			    c_SLWR   <= '0';
			    state <= S_EP2_PKTEND;
		  else 
				ep2_cnt <= ep2_cnt +1;
			    c_SLWR   <= '0';				
				if (ep2_cnt = X"100") then 
				   F1_rdreq <= '0'; 
				   ep2_cnt  <= X"000";	
				   state <= S_idle;
				else 
				   F1_rdreq <= '1';
			    state <= S_EP2_FRD2;				
				end if;
		  end if;		
     when S_EP2_PKTEND =>   -- send partial packet
			c_SLWR   <= '1';
			F1_rdreq <= '0';			
			PKTEND2_cnt <= PKTEND2_cnt +1;
			if ((F1_empty = '0') and (c_EP2FF = '1')) then 
			    state <= S_EP2_FRD ;
			elsif (PKTEND2_cnt = X"5000") THEN  -- wait 16 clocks to see if more data arrives
				c_PKTEND <= '0';
			    ep2_cnt  <= X"000";
				state    <= S_idle;
			else
				state <= S_EP2_PKTEND;
			end if;
			
			
			
			
			when others =>
			
			
			
			
			
     end case;  
end if;
end process;
END behavior;
